/**************************************************************************************

  DC Motor speed and direction of rotation control with PIC16F887 MCU.
  C Code for mikroC PRO for PIC compiler
  Internal oscillator used @ 8MHz
  Configuration words: CONFIG1 = 0x2CD4
                       CONFIG2 = 0x0700
  This is a free software with NO WARRANTY.
  https://simple-circuit.com/

***************************************************************************************/

// direction button connected to pin RB1
#define Button      RB1_bit
// define motor control pins
#define IN1         RD1_bit
#define IN2         RD0_bit


bit direction;
unsigned int motor_speed;

// a small function for Button debounce
char debounce ()
{
  char i, count = 0;
  for(i = 0; i < 5; i++) {
    if (Button == 0)
      count++;
    delay_ms(10);
  }
  if(count > 2)  return 1;
  else           return 0;
}

// main function
void main()
{
  OSCCON = 0x70;  // set internal oscillator to 8MHz
  ANSELH = 0;      // configure all PORTB pins as digital
  PORTD  = 0;      // PORTD initial state
  TRISD  = 0;      // configure all PORTD pins as outputs
  // enable RB1 internal pull up
  NOT_RBPU_bit = 0;      // clear RBPU bit (OPTION_REG.7)
  WPUB         = 0x02;   // WPUB register = 0b00000010
  
  ADC_Init();       // initialize ADC module
  PWM1_Init(1000);  // initialize PWM1 with frequency of 1KHz
  PWM1_Start();     // start PWM1
  
  delay_ms(1000);   // wait 1 second
  direction = 0;

  while(1)
  {
    if(!Button)      // if Button is pressed
    if(debounce())   // call debounce function
    {

      direction = ~direction;  // invert direction variable

      switch (direction)
      {
        case 0: IN1 = 1;
                IN2 = 0;
                break;
        case 1: IN1 = 0;
                IN2 = 1;
      }

      while(debounce());  // call debounce function (wait for Button to be released)

    }

    // read analog data from channel 0 and store it in 'motor_speed'
    motor_speed = ADC_Read(0);
    // set PWM1 duty cycle
    PWM1_Set_Duty(motor_speed >> 2);

    delay_ms(50);   // wait 50 ms
  }

}
// end of code.